import { Injectable, EventEmitter, Component, ElementRef, Output, HostListener, Renderer2, ContentChildren, NgModule } from '@angular/core';
import { BehaviorSubject } from 'rxjs';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * Generated from: lib/services.ts/item.service.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ItemService {
    constructor() {
        this.hasSubmenuSubject = new BehaviorSubject(false);
        this.hasSubmenu$ = this.hasSubmenuSubject.asObservable();
        this.hasWidthSubject = new BehaviorSubject(0);
        this.width$ = this.hasWidthSubject.asObservable();
    }
    /**
     * @param {?} hasWidth
     * @return {?}
     */
    setItemWidth(hasWidth) {
        this.hasWidthSubject.next(hasWidth);
    }
    /**
     * @param {?} hasSubmenu
     * @return {?}
     */
    setHasSubmenu(hasSubmenu) {
        this.hasSubmenuSubject.next(hasSubmenu);
    }
}
ItemService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ItemService.ctorParameters = () => [];
if (false) {
    /**
     * @type {?}
     * @private
     */
    ItemService.prototype.hasSubmenuSubject;
    /** @type {?} */
    ItemService.prototype.hasSubmenu$;
    /**
     * @type {?}
     * @private
     */
    ItemService.prototype.hasWidthSubject;
    /** @type {?} */
    ItemService.prototype.width$;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/controls/context-menu-item.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ContextMenuItemComponent {
    /**
     * @param {?} itemService
     * @param {?} hostEl
     */
    constructor(itemService, hostEl) {
        this.itemService = itemService;
        this.hostEl = hostEl;
        this.activate = new EventEmitter();
        this.lastHoveredSubject = new BehaviorSubject(false);
    }
    /**
     * @return {?}
     */
    get hasSubmenu() {
        return this.hasSubmenuField;
    }
    /**
     * @return {?}
     */
    get lastHovered() {
        return this.lastHoveredField;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.lastHoveredSubscription = this.lastHoveredSubject.subscribe((/**
         * @param {?} _
         * @return {?}
         */
        _ => this.onHoverChange(_)));
        this.hasSubmenuSubscription = this.itemService.hasSubmenu$.subscribe((/**
         * @param {?} _
         * @return {?}
         */
        _ => this.hasSubmenuField = _));
    }
    /**
     * @param {?} _
     * @return {?}
     */
    onHoverChange(_) {
        this.lastHoveredField = _;
        if (this.lastHoveredField) {
            this.itemService.setItemWidth(this.hostEl.nativeElement.offsetWidth);
        }
    }
    /**
     * @return {?}
     */
    onMouseEnter() {
        this.lastHoveredSubject.next(true);
        this.canDeactivate = false;
        this.activate.emit(true);
    }
    /**
     * @return {?}
     */
    onMouseLeave() {
        this.canDeactivate = true;
    }
    /**
     * @return {?}
     */
    setAsInActive() {
        if (this.canDeactivate === false) {
            return;
        }
        this.lastHoveredSubject.next(false);
    }
    /**
     * @return {?}
     */
    hide() {
        this.lastHoveredSubject.next(false);
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.lastHoveredSubject != null) {
            this.lastHoveredSubscription.unsubscribe();
        }
        if (this.hasSubmenuSubscription != null) {
            this.hasSubmenuSubscription.unsubscribe();
        }
    }
}
ContextMenuItemComponent.decorators = [
    { type: Component, args: [{
                // tslint:disable-next-line:component-selector
                selector: '[lib-context-menu-item]',
                template: `<ng-content></ng-content>`
                // tslint:disable-next-line:no-host-metadata-property
                ,
                host: {
                    '[class.context-menu-item]': '1',
                    '[class.center]': '1',
                    '[class.context-menu-hover]': 'lastHovered',
                    '[class.context-menu-visible]': 'lastHovered',
                    '[class.context-menu-submenu]': 'hasSubmenu',
                },
                providers: [ItemService]
            }] }
];
/** @nocollapse */
ContextMenuItemComponent.ctorParameters = () => [
    { type: ItemService },
    { type: ElementRef }
];
ContextMenuItemComponent.propDecorators = {
    activate: [{ type: Output }],
    onMouseEnter: [{ type: HostListener, args: ['mouseenter',] }],
    onMouseLeave: [{ type: HostListener, args: ['mouseleave',] }]
};
if (false) {
    /**
     * @type {?}
     * @private
     */
    ContextMenuItemComponent.prototype.canDeactivate;
    /** @type {?} */
    ContextMenuItemComponent.prototype.activate;
    /** @type {?} */
    ContextMenuItemComponent.prototype.lastHoveredSubject;
    /**
     * @type {?}
     * @private
     */
    ContextMenuItemComponent.prototype.lastHoveredField;
    /**
     * @type {?}
     * @private
     */
    ContextMenuItemComponent.prototype.hasSubmenuField;
    /**
     * @type {?}
     * @private
     */
    ContextMenuItemComponent.prototype.lastHoveredSubscription;
    /**
     * @type {?}
     * @private
     */
    ContextMenuItemComponent.prototype.hasSubmenuSubscription;
    /**
     * @type {?}
     * @private
     */
    ContextMenuItemComponent.prototype.itemService;
    /**
     * @type {?}
     * @private
     */
    ContextMenuItemComponent.prototype.hostEl;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/controls/context-menu.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ContextMenuComponent {
    /**
     * @param {?} element
     * @param {?} renderer
     */
    constructor(element, renderer) {
        this.element = element;
        this.renderer = renderer;
        this.visible = false;
        this.visibleAnimate = false;
        this.subscriptions = [];
    }
    /**
     * @return {?}
     */
    ngAfterContentChecked() {
        if (this.subscriptions.length > 0 || this.directMenus.length === 0) {
            return;
        }
        this.directMenus.toArray().map((/**
         * @param {?} _
         * @return {?}
         */
        _ => this.subscriptions.push(_.activate.subscribe((/**
         * @return {?}
         */
        () => this.clearInactive())))));
    }
    /**
     * @private
     * @return {?}
     */
    clearInactive() {
        this.directMenus.map((/**
         * @param {?} _
         * @return {?}
         */
        _ => _.setAsInActive()));
    }
    /**
     * @param {?} data
     * @param {?} event
     * @return {?}
     */
    setup(data, event) {
        this.data = data;
        this.show(event.clientX, event.clientY);
        event.preventDefault();
        event.stopPropagation();
        return false;
    }
    /**
     * @param {?} x
     * @param {?} y
     * @return {?}
     */
    show(x, y) {
        this.renderer.setStyle(this.element.nativeElement, 'left', `${x}px`);
        this.renderer.setStyle(this.element.nativeElement, 'top', `${y}px`);
        this.visible = true;
        setTimeout((/**
         * @return {?}
         */
        () => this.visibleAnimate = true));
    }
    /**
     * @return {?}
     */
    hide() {
        this.visibleAnimate = false;
        setTimeout((/**
         * @return {?}
         */
        () => this.visible = false), 300);
        this.menus.map((/**
         * @param {?} _
         * @return {?}
         */
        _ => _.hide()));
    }
    /**
     * @return {?}
     */
    onClick() {
        this.hide();
    }
    /**
     * @param {?} ev
     * @return {?}
     */
    onContextMenu(ev) {
        ev.preventDefault();
        ev.stopPropagation();
        return false;
    }
    /**
     * @return {?}
     */
    onScroll() {
        this.hide();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.subscriptions.map((/**
         * @param {?} _
         * @return {?}
         */
        _ => _.unsubscribe()));
    }
}
ContextMenuComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-context-menu',
                template: `
      <nav class="context-menu" tabindex="-1" [ngClass]="{'in': visibleAnimate}"
      [ngStyle]="{'opacity': visibleAnimate ? 1 : 0, 'background-color':'white'}">
        <ul class="context-menu-list context-menu-root">
          <ng-content></ng-content>
        </ul>
      </nav>
  `,
                // tslint:disable-next-line:no-host-metadata-property
                host: {
                    '[class.contextmenu-container]': '1',
                    '[class.show]': 'visible',
                    '[class.hide]': '!visible',
                },
                styles: ["@charset \"UTF-8\";:host ::ng-deep .context-menu-icon-add:before{content:\"\uEA01\"}:host ::ng-deep .context-menu-icon-copy:before{content:\"\uEA02\"}:host ::ng-deep .context-menu-icon-cut:before{content:\"\uEA03\"}:host ::ng-deep .context-menu-icon-delete:before{content:\"\uEA04\"}:host ::ng-deep .context-menu-icon-edit:before{content:\"\uEA05\"}:host ::ng-deep .context-menu-icon-paste:before{content:\"\uEA06\"}:host ::ng-deep .context-menu-icon-quit:before{content:\"\uEA07\"}:host ::ng-deep .context-menu-icon::before{position:absolute;top:50%;left:0;width:2em;font-family:context-menu-icons;font-size:1em;font-style:normal;font-weight:400;line-height:1;color:#2980b9;text-align:center;transform:translateY(-50%);-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}:host ::ng-deep .context-menu-icon.context-menu-hover:before{color:#fff}:host ::ng-deep .context-menu-icon.context-menu-disabled::before{color:#bbb}:host ::ng-deep .context-menu-icon.context-menu-icon--fa{display:list-item}:host ::ng-deep .context-menu-icon.context-menu-icon--fa::before{position:absolute;top:50%;left:0;width:2em;font-family:FontAwesome;font-size:1em;font-style:normal;font-weight:400;line-height:1;color:#2980b9;text-align:center;transform:translateY(-50%);-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}:host ::ng-deep .context-menu-icon.context-menu-icon--fa.context-menu-hover:before{color:#fff}:host ::ng-deep .context-menu-icon.context-menu-icon--fa.context-menu-disabled::before{color:#bbb}:host ::ng-deep .context-menu-list{position:absolute;display:inline-block;min-width:13em;max-width:26em;padding:.25em 0;margin:.03em;font-family:inherit;font-size:inherit;list-style-type:none;background:#fff;border:1px solid #bebebe;border-radius:.2em;box-shadow:0 2px 5px rgba(0,0,0,.5)}:host ::ng-deep .context-menu-item{position:relative;padding:.2em 2em;color:#2f2f2f;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;background-color:#fff}:host ::ng-deep .context-menu-separator{padding:0;margin:.35em 0;border-bottom:1px solid #e6e6e6}:host ::ng-deep .context-menu-item>label>input,:host ::ng-deep .context-menu-item>label>textarea{-webkit-user-select:text;-moz-user-select:text;-ms-user-select:text;user-select:text}:host ::ng-deep .context-menu-item.context-menu-hover{color:#fff;cursor:pointer;background-color:#2980b9}:host ::ng-deep .context-menu-item.context-menu-disabled{color:#bbb;cursor:default;background-color:#fff}:host ::ng-deep .context-menu-input.context-menu-hover{cursor:default}:host ::ng-deep .context-menu-submenu:after{position:absolute;top:50%;right:.5em;z-index:1001;width:0;height:0;content:\"\";border-color:transparent transparent transparent #2f2f2f;border-style:solid;border-width:.25em 0 .25em .25em;transform:translateY(-50%)}:host ::ng-deep .context-menu-item.context-menu-input{padding:.3em .6em}:host ::ng-deep .context-menu-input>label>*{vertical-align:top}:host ::ng-deep .context-menu-input>label>input[type=checkbox],:host ::ng-deep .context-menu-input>label>input[type=radio]{position:relative;top:.12em;margin-right:.4em}:host ::ng-deep .context-menu-input>label{margin:0}:host ::ng-deep .context-menu-input>label,:host ::ng-deep .context-menu-input>label>input[type=text],:host ::ng-deep .context-menu-input>label>select,:host ::ng-deep .context-menu-input>label>textarea{display:block;width:100%;box-sizing:border-box}:host ::ng-deep .context-menu-input>label>textarea{height:7em}:host ::ng-deep .context-menu-item>.context-menu-list{top:.3em;right:-.3em;display:none}:host ::ng-deep .context-menu-item.context-menu-visible>.context-menu-list{display:block}:host ::ng-deep .context-menu-accesskey{text-decoration:underline}", `:host(.hide) {
      display: none;
    }
    :host(.contextmenu-container) {
      position: fixed;
      z-index: 1000;
    }
    :host(.show) {
      display: block;
    }
  }`]
            }] }
];
/** @nocollapse */
ContextMenuComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
ContextMenuComponent.propDecorators = {
    menus: [{ type: ContentChildren, args: [ContextMenuItemComponent, { descendants: true },] }],
    directMenus: [{ type: ContentChildren, args: [ContextMenuItemComponent, { descendants: false },] }],
    onClick: [{ type: HostListener, args: ['document:click', [],] }],
    onContextMenu: [{ type: HostListener, args: ['contextmenu', ['$event.target'],] }],
    onScroll: [{ type: HostListener, args: ['document:scroll', [],] }]
};
if (false) {
    /** @type {?} */
    ContextMenuComponent.prototype.visible;
    /** @type {?} */
    ContextMenuComponent.prototype.visibleAnimate;
    /** @type {?} */
    ContextMenuComponent.prototype.data;
    /**
     * @type {?}
     * @private
     */
    ContextMenuComponent.prototype.subscriptions;
    /**
     * @type {?}
     * @private
     */
    ContextMenuComponent.prototype.menus;
    /**
     * @type {?}
     * @private
     */
    ContextMenuComponent.prototype.directMenus;
    /**
     * @type {?}
     * @private
     */
    ContextMenuComponent.prototype.element;
    /**
     * @type {?}
     * @private
     */
    ContextMenuComponent.prototype.renderer;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/controls/context-sub-menu.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ContextSubMenuComponent {
    /**
     * @param {?} parentItem
     * @param {?} hostEl
     * @param {?} renderer
     */
    constructor(parentItem, hostEl, renderer) {
        this.parentItem = parentItem;
        this.hostEl = hostEl;
        this.renderer = renderer;
        this.childSubscriptions = [];
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.widthSubscription = this.parentItem.width$.subscribe((/**
         * @param {?} _
         * @return {?}
         */
        _ => this.setPosition(_)));
        this.parentItem.setHasSubmenu(true);
    }
    /**
     * @param {?} _
     * @return {?}
     */
    setPosition(_) {
        this.renderer.setStyle(this.hostEl.nativeElement, 'top', '0px');
        this.renderer.setStyle(this.hostEl.nativeElement, 'left', `${_ - 1}px`);
        this.renderer.removeStyle(this.hostEl.nativeElement, 'right');
    }
    /**
     * @return {?}
     */
    ngAfterContentChecked() {
        if (this.childSubscriptions.length > 0 || this.directMenus.length === 0) {
            return;
        }
        this.directMenus.toArray().map((/**
         * @param {?} _
         * @return {?}
         */
        _ => this.childSubscriptions.push(_.activate.subscribe((/**
         * @return {?}
         */
        () => this.clearInactive())))));
    }
    /**
     * @private
     * @return {?}
     */
    clearInactive() {
        this.directMenus.map((/**
         * @param {?} _
         * @return {?}
         */
        _ => _.setAsInActive()));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.childSubscriptions.map((/**
         * @param {?} _
         * @return {?}
         */
        _ => _.unsubscribe()));
        this.widthSubscription.unsubscribe();
    }
}
ContextSubMenuComponent.decorators = [
    { type: Component, args: [{
                // tslint:disable-next-line:component-selector
                selector: '[lib-context-submenu]',
                template: `<ng-content></ng-content>`,
                // tslint:disable-next-line:no-host-metadata-property
                host: {
                    '[class.context-menu-list]': '1',
                }
            }] }
];
/** @nocollapse */
ContextSubMenuComponent.ctorParameters = () => [
    { type: ItemService },
    { type: ElementRef },
    { type: Renderer2 }
];
ContextSubMenuComponent.propDecorators = {
    directMenus: [{ type: ContentChildren, args: [ContextMenuItemComponent, { descendants: false },] }]
};
if (false) {
    /**
     * @type {?}
     * @private
     */
    ContextSubMenuComponent.prototype.directMenus;
    /**
     * @type {?}
     * @private
     */
    ContextSubMenuComponent.prototype.childSubscriptions;
    /**
     * @type {?}
     * @private
     */
    ContextSubMenuComponent.prototype.widthSubscription;
    /**
     * @type {?}
     * @private
     */
    ContextSubMenuComponent.prototype.parentItem;
    /**
     * @type {?}
     * @private
     */
    ContextSubMenuComponent.prototype.hostEl;
    /**
     * @type {?}
     * @private
     */
    ContextSubMenuComponent.prototype.renderer;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/controls/context-menu-separator.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ContextMenuSeparatorComponent {
}
ContextMenuSeparatorComponent.decorators = [
    { type: Component, args: [{
                // tslint:disable-next-line:component-selector
                selector: '[lib-context-menu-separator]',
                template: `<li class="context-menu-separator"></li>`
            }] }
];

/**
 * @fileoverview added by tsickle
 * Generated from: public-api.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ContextMenuModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: ContextMenuModule,
            providers: []
        };
    }
}
ContextMenuModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [
                    ContextMenuComponent, ContextMenuItemComponent, ContextSubMenuComponent, ContextMenuSeparatorComponent,
                ],
                exports: [
                    ContextMenuComponent, ContextMenuItemComponent, ContextSubMenuComponent, ContextMenuSeparatorComponent,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * Generated from: controls-context-menu.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ContextMenuComponent, ContextMenuModule, ContextMenuItemComponent as ɵa, ItemService as ɵb, ContextSubMenuComponent as ɵc, ContextMenuSeparatorComponent as ɵd };
//# sourceMappingURL=controls-context-menu.js.map
